//
//  BuildResultParser.m
//  ActionBuild
//
//  Created by Mike Murray on 2/23/10.
//  Copyright 2010 Wizaado. All rights reserved.
//

#import "BuildResultParser.h"


@implementation BuildResultParser

- (id) init {
	
	self = [super init];
	
	mxmlcErrorType = [[OnigRegexp compile:@"(Error|Warning):"] retain];
	mxmlcErrorMessage = [[OnigRegexp compile:@"(/.*?.[as|mxml|swf]+).([0-9]+).: ((.*?[ .])+)"] retain];
	
	errorCount = 0;
	warningCount = 0;
	
	return self;
}

- (void)dealloc {
	
	[mxmlcErrorType release];
	[mxmlcErrorMessage release];
	
	[super dealloc];
}

- (void)reset {
	errorCount = 0;
	warningCount = 0;
}

- (OutputMessage*) outputMessageFromData:(NSData*)data {
	
	NSString *stdOutout= [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease];		
	OnigResult *regexpResult = [mxmlcErrorType search:stdOutout];
	
	OutputMessage *m = [[OutputMessage alloc] init];
	m.fullMessage = stdOutout;
	
	
	if([[regexpResult stringAt:1] isEqual:@"Error"]) {
		errorCount += 1;			
		regexpResult = [mxmlcErrorMessage search:stdOutout];
		
		m.type = @"Error";
		m.line = [regexpResult stringAt:2];
		m.message = [regexpResult stringAt:3];
		m.file = [[regexpResult stringAt:1] lastPathComponent];
		return m;
		
	} else if ([[regexpResult stringAt:1] isEqual:@"Warning"]) {
		warningCount += 1;
		
		m.type = @"Warning";
		m.message = stdOutout;
		return m;
	} else {
		//[controller appendOutput:stdOutout];
	}
	
	return nil;
}

- (int) compileErrorCount {
	return errorCount;
}

- (int) compileWarningCount {
	return warningCount;
}

@end
