//
//  fcshTask.m
//  ActionBuild
//
//  Created by Michael Murray on 8/14/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "BuildTask.h"
#import "ProjectController.h"

@implementation BuildTask


//- (id)initWithController:(id <BuildTaskController>)aController andProjectModel:(NSDictionary*)pdata
- (id)initWithController:(id <BuildTaskController>)aController {
    self = [super init];
	
	argumentID = 0;
	compileErrors = 0;
	controller = aController;
	
	return self;
}

- (void)dealloc
{
    [self stopProcess];
	
	[compilerArguments release];
    [launchTask release];
	[messageTypeRegexp release];
	[errorMessageRegexp release];
	
    [super dealloc];
}

- (NSString *) createPathToFile:(NSString *)file {
	
	if([file isAbsolutePath])
		return file;
	
	return [[project.projectFilePath stringByAppendingFormat:@"/%@", file] stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
}

- (NSArray *) makeMultipleArguments:(NSArray *)arrayOfTableItems {
	NSMutableArray *finalOutput = [[NSMutableArray alloc] init];
	
	for(TableItem *ti in arrayOfTableItems) {
		[finalOutput addObject:ti.item];
	}
	NSLog(@"%@", finalOutput);
	return finalOutput;
}

/*
 * Make Arguments 
 *
 */
- (NSArray *) makeMultipleArguments:(NSArray *)arrayOfTableItems withPrefix:(NSString *)prefix {
	
	NSMutableArray *finalOutput = [[[NSMutableArray alloc] init] autorelease];
	
	for(TableItem *ti in arrayOfTableItems) {
		[finalOutput addObject:[NSString stringWithFormat:@"%@%@", prefix, ti.item]];
	}
	
	return finalOutput;
}

- (void) buildUsingFlashCS4:(NSString*)directory projectData:(NSDictionary*)projectData {
	
	compiler = flashcommand;
	
	messageTypeRegexp = [[OnigRegexp compile:@"(Error|Warning)"] retain];
	errorMessageRegexp = [[OnigRegexp compile:@"..(Error)...([a-zA-Z0-9\', ]+):.([0-9]+):.([a-zA-Z0-9 .]+)."] retain];
	
	
	NSString *flashcommand = [[NSBundle mainBundle] pathForResource:@"flashcommand" ofType:nil];
	
	compileErrors = 0;
	compileWarnings = 0;
	
	[controller setOutput:@"Building...\n"];
	[controller clearMessages];
	
	if(launchTask) {
		[launchTask release];
		launchTask = nil;
	}
	
	launchTask = [[NSTask alloc] init];
	[launchTask setStandardOutput: [NSPipe pipe]];
	[launchTask setStandardError: [launchTask standardOutput]];
	
	NSMutableArray *args = [[NSMutableArray alloc] initWithCapacity:2];
	NSString *sourceFile = [[projectData objectForKey:@"source"] retain];
	NSString *outputFile = [[projectData objectForKey:@"output"] retain];
	directory = [directory stringByDeletingLastPathComponent];
	
	if(![sourceFile isAbsolutePath]) {
		sourceFile = [directory stringByAppendingPathComponent:[projectData objectForKey:@"source"]];
	}
	
	if(![outputFile isAbsolutePath]) {
		outputFile = [directory stringByAppendingPathComponent:[projectData objectForKey:@"source"]];
	}
	
	[args addObject: @"-e"];
	[args addObject: @"-s"];
	[args addObject:[self createPathToFile:sourceFile]];
	[args addObject: @"-o"];
	[args addObject:[self createPathToFile:sourceFile]];
	
	
	
	[self launchProcessStarted];
	
	[launchTask setLaunchPath: flashcommand];
	[launchTask setArguments: args];
	
	[[NSNotificationCenter defaultCenter] addObserver:self 
											 selector:@selector(getData:) 
												 name: NSFileHandleReadCompletionNotification 
											   object: [[launchTask standardOutput] fileHandleForReading]];
	
	
	[[[launchTask standardOutput] fileHandleForReading] readInBackgroundAndNotify];
	
	// launch the task asynchronously
	[launchTask launch]; 
	
}

- (void) buildUsingMXMLC:(NSString*)directory projectData:(NSDictionary*)projectData {
	
	//projectData = [pdata retain];
	
	
	
	compiler = mxmlc;
	
	// Regular Expressions
	messageTypeRegexp = [[OnigRegexp compile:@"(Error|Warning):"] retain];
	errorMessageRegexp = [[OnigRegexp compile:@"(/.*?.[as|mxml|swf]+).([0-9]+).: ((.*?[ .])+)"] retain];
	
	// Compiler Error / Warnings
	compileErrors = 0;
	compileWarnings = 0;
	
	// Messages
	//[controller setOutput:@"Building...\n"];
	//[controller clearMessages];
	
	// Launch Task
	if(launchTask) {
		[launchTask release];
		launchTask = nil;
	}
			
	launchTask = [[NSTask alloc] init];
	[launchTask setStandardOutput: [NSPipe pipe]];
	[launchTask setStandardError: [launchTask standardOutput]];
	
	
	NSLog(@"%@", projectData);
	
	// Compiler Argument Building
	NSMutableArray *args = [[NSMutableArray alloc] initWithCapacity:2];

	// Source / Output
	NSString *sourceFile = [[projectData objectForKey:@"source"] retain];
	NSString *outputFile = [[projectData objectForKey:@"output"] retain];
	directory = [directory stringByDeletingLastPathComponent];
	
	if(![sourceFile isAbsolutePath]) {
		sourceFile = [directory stringByAppendingPathComponent:[projectData objectForKey:@"source"]];
	}
	
	if(![outputFile isAbsolutePath]) {
		outputFile = [directory stringByAppendingPathComponent:[projectData objectForKey:@"source"]];
	}
	
	[args addObject: [NSString stringWithFormat:@"%@", sourceFile]];
	[args addObject: [NSString stringWithFormat:@"-output=%@", outputFile]];
	
	//[sourceFile release];
	//[outputFile release];
	
	// MXMLC flags
	/*
	NSArray *flags = [projectData objectForKey:@"flags"];
	if([flags count] > 0) {
		NSArray *margs = [self makeMultipleArguments:flags];
		[args addObjectsFromArray:margs];
	}
	*/
	[self launchProcessStarted];
	
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
	
	NSString *sdkPath;
	
	//if(![self.project isAIRProject])
		sdkPath = [[userDefaults objectForKey:@"flexSDKPath"] stringByAppendingPathComponent:@"bin/mxmlc"];
	//else {
	//	sdkPath = [[userDefaults objectForKey:@"flexSDKPath"] stringByAppendingPathComponent:@"bin/amxmlc"];
	//}
	 
	
	[launchTask setArguments: args];
	[launchTask setLaunchPath: sdkPath];
	
	
	[[NSNotificationCenter defaultCenter] addObserver:self 
											 selector:@selector(getData:) 
												 name: NSFileHandleReadCompletionNotification 
											   object: [[launchTask standardOutput] fileHandleForReading]];
	
	
	[[[launchTask standardOutput] fileHandleForReading] readInBackgroundAndNotify];

	// launch the task asynchronously
	[launchTask launch]; 
	
}



- (NSMutableArray *) compilerArguments {
	return compilerArguments;
}

- (void) setCompilerArguments:(NSMutableArray *)value {
	if (compilerArguments !=nil) {
		for (id obj in compilerArguments) {
			[obj release];
		}
		//[recs release] if i uncomment this it crashes
	}
	
	compilerArguments = [value retain];
	
}

// If the task ends, there is no more data coming through the file handle even when the notification is
// sent, or the process object is released, then this method is called.
- (void) stopProcess
{
	/*    // we tell the controller that we finished, via the callback, and then blow away our connection
	 // to the controller.  NSTasks are one-shot (not for reuse), so we might as well be too.
	 [controller processFinished];
	 controller = nil;*/
    NSData *data;
    
    // It is important to clean up after ourselves so that we don't leave potentially deallocated
    // objects as observers in the notification center; this can lead to crashes.
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object: [[launchTask standardOutput] fileHandleForReading]];
    
    // Make sure the task has actually stopped!
    [launchTask terminate];
	
	while ((data = [[[launchTask standardOutput] fileHandleForReading] availableData]) && [data length])
	{
		
		[controller appendOutput: [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease]];
	}
	
	// we tell the controller that we finished, via the callback, and then blow away our connection
	// to the controller.  NSTasks are one-shot (not for reuse), so we might as well be too.
	[self launchProcessEnded];
}

// This method is called asynchronously when data is available from the task's file handle.
// We just pass the data along to the controller as an NSString.
- (void) getData: (NSNotification *)aNotification
{
    NSData *data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
	
	if ([data length])  {
		
		NSString *stdOutout= [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease];		
		OnigResult *regexpResult = [messageTypeRegexp search:stdOutout];
		
		OutputMessage *m = [[OutputMessage alloc] init];
		m.fullMessage = stdOutout;
				
		switch (compiler) {
			case mxmlc:
				if([[regexpResult stringAt:1] isEqual:@"Error"]) {
					compileErrors += 1;			
					regexpResult = [errorMessageRegexp search:stdOutout];
					
					m.type = @"Error";
					m.line = [regexpResult stringAt:2];
					m.message = [regexpResult stringAt:3];
					m.file = [[regexpResult stringAt:1] lastPathComponent];
					[controller addMessage:m];
					
				} else if ([[regexpResult stringAt:1] isEqual:@"Warning"]) {
					compileWarnings += 1;
					
					m.type = @"Warning";
					m.message = stdOutout;
					[controller addMessage:m];
				} else {
					[controller appendOutput:stdOutout];
				}
				break;
			case flashcommand:
				
				NSLog(@"It didnt work: %@", [regexpResult strings]);
				
				if([[regexpResult stringAt:1] isEqual:@"Error"]) {
					compileErrors += 1;			
					regexpResult = [errorMessageRegexp search:stdOutout];
					
					m.type = @"Error";
					m.line = [regexpResult stringAt:2];
					m.message = [regexpResult stringAt:4];
					m.file = [[regexpResult stringAt:1] lastPathComponent];
					[controller addMessage:m];
					
					NSLog(@"Fcmd %@", [regexpResult strings]);
					
				} else if ([[regexpResult stringAt:1] isEqual:@"Warning"]) {
					compileWarnings += 1;
					
					m.type = @"Warning";
					m.message = stdOutout;
					[controller addMessage:m];
				} else {
					//[controller appendOutput:stdOutout];
				}
				
				[controller appendOutput:stdOutout];
				break;

			default:
				
				break;
		}
		

		
		
		/*
		NSPredicate *errorCheck = [NSPredicate predicateWithFormat:@"SELF CONTAINS[cd] %@", @"Error"];
		if ([errorCheck evaluateWithObject:stdOutout] == YES) {
			
			messageLevel = @"Error";
			m.type = @"Error";
		}
		
		NSPredicate *warningCheck = [NSPredicate predicateWithFormat:@"SELF CONTAINS[cd] %@", @"Warning"];
		if ([warningCheck evaluateWithObject:stdOutout] == YES) {
			compileWarnings += 1;
			messageLevel = @"Warning";
			m.type = @"Warning";
		}
		*/
		
		
		
		
        
    } else {
        // We're finished here
        [self stopProcess];
    }
    
    // we need to schedule the file handle go read more data in the background again.
    [[aNotification object] readInBackgroundAndNotify];  
}

- (void) launchStop {
	
}
- (void) launchProcessStarted {
	
}
- (void) launchProcessEnded {
	
	NSMutableString *finalOutput = [[NSMutableString alloc] initWithString:@"Build"];
	
	if(compileErrors == 0) {
		[finalOutput appendString:@" completed"];
	}
	else if(compileErrors == 1) {
		[finalOutput appendFormat:@" failed with %d error", compileErrors];
	}
	else {
		[finalOutput appendFormat:@" failed with %d error(s)", compileErrors];
	}
		
	if(compileWarnings > 0) {
		if(compileErrors == 0) {
			if(compileWarnings == 1) {
				[finalOutput appendFormat:@" with %d warning.", compileWarnings];
				
			} else {
				[finalOutput appendFormat:@" with %d warning(s).", compileWarnings];
			}
		}
		else {
			if(compileWarnings == 1) {
				[finalOutput appendFormat:@" and %d warning.", compileWarnings];
				
			} else {
				[finalOutput appendFormat:@" and %d warning(s).", compileWarnings];
			}
		}
	
	}
	[controller setStatus:[finalOutput copy]];
	
	
	// Successful Build
	if(compileErrors == 0) {
		[controller processFinished];
	} else {
		[controller processFinishedWithErrors];
	}
	
	controller = nil;
	
	
}

@end
