//
//  fcshTask.m
//  ActionBuild
//
//  Created by Michael Murray on 8/14/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "FlexDebugger.h"


@implementation FlexDebugger

- (id)initWithController:(id <FlexDebuggerController>)aController
{
    self = [super init];
    // Add any code here that needs to be executed once the windowController has loaded the document's window.
	
	//regextest = [[NSPredicate alloc] init];
	
	debuggerRunning = NO;
	controller = aController;
	
	return self;
}

-(void) launch:(NSString*)path withDebugger:(BOOL)debug {
	
	
	[controller setOutput:@"Running Flex debugger\n"];
	
	if(launchTask) {
		
		if(debuggerRunning)
			[self quitFlexDebugger];
		
		[launchTask release];
		launchTask = nil;
		
	}
		
	launchTask = [[NSTask alloc] init];
	pipe = [[NSPipe alloc] init];
	inputPipe = [[NSPipe alloc] init];
	
	[launchTask setStandardOutput: pipe];
	[launchTask setStandardError: [launchTask standardOutput]];
	[launchTask setStandardInput:inputPipe];
	
	inputHandler = [inputPipe fileHandleForWriting];
	
	NSMutableArray *args = [NSMutableArray arrayWithCapacity:3];	
	
	if(debug == NO) {
		[args addObjectsFromArray:[NSArray arrayWithObjects:@"/usr/bin/open", path, nil]];
		debuggerRunning = NO;
	}
	else {
		[args addObjectsFromArray:[NSArray arrayWithObjects:@"/Developer/SDKs/flex_sdk_3/bin/fdb", path ,nil]];
		debuggerRunning = YES;
	}
	
	[self launchProcessStarted];
	
	[launchTask setLaunchPath: [args objectAtIndex:0]];
	[launchTask setArguments: [args subarrayWithRange: NSMakeRange (1, ([args count] - 1))]];
	
	[[NSNotificationCenter defaultCenter] addObserver:self 
											 selector:@selector(getData:) 
												 name: NSFileHandleReadCompletionNotification 
											   object: [[launchTask standardOutput] fileHandleForReading]];
	
	[[[launchTask standardOutput] fileHandleForReading] readInBackgroundAndNotify];
	
	// launch the task asynchronously
	[launchTask launch]; 
	
	if(debug == YES)
		[inputHandler writeData:[@"c\r" dataUsingEncoding:NSUTF8StringEncoding]];
	
}


- (void) quitFlexDebugger {
	if (debuggerRunning) {
		[inputHandler writeData:[@"quit\r" dataUsingEncoding:NSUTF8StringEncoding]];
	}
}

// If the task ends, there is no more data coming through the file handle even when the notification is
// sent, or the process object is released, then this method is called.
- (void) stopProcess
{
	/*    // we tell the controller that we finished, via the callback, and then blow away our connection
	 // to the controller.  NSTasks are one-shot (not for reuse), so we might as well be too.
	 [controller processFinished];
	 controller = nil;*/
    NSData *data;
    
    // It is important to clean up after ourselves so that we don't leave potentially deallocated
    // objects as observers in the notification center; this can lead to crashes.
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object: [[launchTask standardOutput] fileHandleForReading]];
    
    // Make sure the task has actually stopped!
    [launchTask terminate];
	
	while ((data = [[[launchTask standardOutput] fileHandleForReading] availableData]) && [data length])
	{
		
		[controller appendOutput: [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease]];
	}
	
	// we tell the controller that we finished, via the callback, and then blow away our connection
	// to the controller.  NSTasks are one-shot (not for reuse), so we might as well be too.
	[self launchProcessEnded];
}

// This method is called asynchronously when data is available from the task's file handle.
// We just pass the data along to the controller as an NSString.
- (void) getData: (NSNotification *)aNotification
{
    NSData *data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
    // If the length of the data is zero, then the task is basically over - there is nothing
    // more to get from the handle so we may as well shut down.
    if ([data length])
    {
        // Send the data on to the controller; we can't just use +stringWithUTF8String: here
        // because -[data bytes] is not necessarily a properly terminated string.
        // -initWithData:encoding: on the other hand checks -[data length]
		
		
		NSString *stdOutout= [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease];		
		/*
		regextest = [NSPredicate predicateWithFormat:@"SELF CONTAINS[cd] %@", @"fcsh: Assigned 1 as the compile target id"];
		if ([regextest evaluateWithObject:stdOutout] == YES) {
			argumentID = 1;		
			NSLog(@"Set argument ID");
		}
		NSPredicate *errorCheck = [NSPredicate predicateWithFormat:@"SELF CONTAINS[cd] %@", @"Error"];
		if ([errorCheck evaluateWithObject:stdOutout] == YES) {
			compileErrors += 1;
			NSLog(@"Set compile Errors:");
		}
		
		regextest = [NSPredicate predicateWithFormat:@"SELF CONTAINS[cd] %@", @"(fcsh)"];
		if ([regextest evaluateWithObject:stdOutout] == YES) {
			[self launchProcessEnded];
			NSLog(@"DONE!!!!");
		}
		*/
        [controller appendOutput: stdOutout];
		//NSLog(@"%@", [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease]);
    } else {
        // We're finished here
        [self launchStop];
    }
    
    // we need to schedule the file handle go read more data in the background again.
    [[aNotification object] readInBackgroundAndNotify];  
}

- (void) launchStop {
	
}
- (void) launchProcessStarted {
	
}
- (void) launchProcessEnded {
	//[self quitFlexDebugger];
	[controller processFinished];
}

@end
