//
//  ProjectController.h
//  ActionBuild
//
//  Created by Michael Murray on 12/25/09.
//  Copyright 2009 Wizaado. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <BWToolkitFramework/BWToolkitFramework.h>
#import "TableItem.h"
#import "ProjectListItem.h"
#import "BuildTask.h"
#import "BuildViewController.h"
#import "BuildResultParser.h"
#import "RunViewController.h"

typedef enum {
	FLASH = 1001, 
	AIR = 1002
} ProjectType;

typedef enum {
	BUILTIN = 1001, 
	EXTERNAL = 1002, 
} PreviewBrowser;


@interface ProjectController : NSObject<BuildTaskController> {
	
	NSWindow *window;
	
	NSTask *buildTask;
	
	NSMenu *projectMenu;
	
	NSMutableDictionary *sugarDefaults;
	NSMutableArray *openedProjects;
	NSArrayController *projectsList;
	
	NSMutableDictionary *projectData;
	
	NSPopUpButton *projectTypePopUp;
	NSPopUpButton *previewBrowserPopUp;
	NSTextField *configField;
	NSTextField *sourceField;
	NSTextField *outputField;
	NSTextField *launchField;
	NSArrayController *compileFlagsController;
	
	// Project Info
	NSString *projectFilePath;
	
	//BuildTask *buildTask;
	
	BOOL compilerRunning;
	BOOL debug;
	
	IBOutlet NSTextView *outputText;
	IBOutlet NSTextField *statusLabel;
	
	BuildResultParser *bsr;
	
	BuildViewController *buildView;
	RunViewController *runView;
	NSTabView *mainTabView;
}

- (void) addProjectToUserDefaults:(NSString*)filePath;
- (IBAction)openProjectSettings:(id)sender;
- (void)projectWithContentsOfFile:(NSString*)filePath;
- (IBAction) projectsListSelectionChange:(id)sender;
- (id)objectForKey:(NSString*)key;

- (BOOL) settingsAreValid;
- (void)saveProject;
- (NSURL*) urlToFileInProject:(NSString*)path;
- (NSString*) absolutePathToFileInProject:(NSString*)path;

@property(nonatomic, retain) BuildViewController *buildView;
@property(nonatomic, retain) RunViewController *runView;
@property(nonatomic, retain) NSTabView *mainTabView;

@property(nonatomic, retain) IBOutlet NSPopUpButton *projectTypePopUp;
@property(nonatomic, retain) IBOutlet NSPopUpButton *previewBrowserPopUp;
@property(nonatomic, retain) IBOutlet NSTextField *configField;
@property(nonatomic, retain) IBOutlet NSTextField *sourceField;
@property(nonatomic, retain) IBOutlet NSTextField *outputField;
@property(nonatomic, retain) IBOutlet NSTextField *launchField;
@property(nonatomic, retain) IBOutlet NSArrayController *compileFlagsController;
@property(nonatomic, retain) IBOutlet NSMenu *projectMenu;

@property (nonatomic,retain) NSString *projectFilePath;
@property (readwrite,assign) NSInteger projectType;
@property (readwrite,assign) NSInteger previewBrowser;
@property (readwrite,assign) NSString *configFile;
@property (readwrite,assign) NSString *sourceFile;
@property (readwrite,assign) NSString *outputFile;
@property (readwrite,assign) NSString *launchFile;
@property (readwrite,assign) NSArrayController *flags;
@property (nonatomic,retain) NSMutableDictionary *projectData;

@property (nonatomic, retain) IBOutlet NSArrayController *projectsList;

- (IBAction) debugProject:(id)sender;
- (IBAction) runProject:(id)sender;
- (void) launchProject;
- (void) stopBuildProcess;
-(NSString*) projectDirectory;
@end
