//
//  ProjectController.m
//  ActionBuild
//
//  Created by Michael Murray on 12/25/09.
//  Copyright 2009 Wizaado. All rights reserved.
//

#import "ProjectController.h"
#import "TableItem.h"

@implementation ProjectController

NSString *userDefaultsProjects = @"com.wizaado.FlashSugar.projects";

@synthesize projectsList;
@synthesize projectFilePath;
@synthesize buildView;
@synthesize runView;
@synthesize mainTabView;

-(id) init {
	self = [super init];
		
	bsr = [BuildResultParser new];
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	
	if([defaults objectForKey:@"Projects"] == nil) {
		[defaults setObject:[NSDictionary dictionary] forKey:@"Projects"];
		
	}
	
	openedProjects = [defaults objectForKey:@"Projects"];
	
	
	/*
	if(![defaults objectForKey:userDefaultsProjects]) {
		
		NSArray *array = [NSArray new];
		[defaults setObject:array forKey:userDefaultsProjects];
		[array release];
		
	}*/
	
	//NSUserDefaults *defaults
	
	return self;
}

- (IBAction) projectsListSelectionChange:(id)sender {
	
	[projectsList setSelectionIndex:[sender indexOfSelectedItem]];
	
	ProjectListItem *p = [[projectsList selectedObjects] objectAtIndex:0];
	[self projectWithContentsOfFile:p.filePath];
	
}

- (void) awakeFromNib {

}

- (void) addProjectToUserDefaults:(NSString*)filePath {
	
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	
	if(openedProjects == nil) {
		[defaults setObject:[NSArray array] forKey:@"Projects"];
	}
	
	NSMutableArray *projects = [[defaults objectForKey:@"Projects"] mutableCopy];
	
	if([projects containsObject:filePath] == NO) {
		[projects addObject:filePath];
		[defaults setObject:projects forKey:@"Projects"];
		
		ProjectListItem *pl = [ProjectListItem new];
		pl.name = [filePath lastPathComponent];
		pl.filePath = filePath;
		
		/*
		[projectsList addObject:pl];
		[pl release];
		*/
	}
	
	
}

- (NSArray*) projectsFromUserDefaults {
	//NSUserDefaults *defaults = [[NSUserDefaults standardUserDefaults] mutableCopy];
	NSArray *projects = [sugarDefaults objectForKey:@"Projects"];
	
	return projects;
}

- (IBAction)openProjectSettings:(id)sender {
	
}



- (void)projectWithContentsOfFile:(NSString*)filePath {
	
	[self addProjectToUserDefaults:filePath];
	
	self.projectFilePath = filePath;
	
	projectData = [[NSMutableDictionary dictionaryWithContentsOfFile:filePath] retain];
	
	
	self.projectType = [[projectData objectForKey:@"project-type"] intValue];
	self.previewBrowser = [[projectData objectForKey:@"preview-browser"] intValue];
	
	
	self.sourceFile = [projectData objectForKey:@"source"];
	self.outputFile = [projectData objectForKey:@"output"];
	self.launchFile = [projectData objectForKey:@"launch"];
	
	//[compileFlagsController removeObjects:[compileFlagsController arrangedObjects]];
	[[compileFlagsController content] removeAllObjects];
	
	for(NSString *flag in [projectData objectForKey:@"flags"]) {
		
		TableItem *ti = [[TableItem alloc] initWithString:flag];
		[compileFlagsController addObject:ti];
		[ti release];
	}

}


- (void)saveProject {
	
	// Project Type
	[projectData setObject:[NSNumber numberWithInt:self.projectType] forKey:@"project-type"];
	
	// Preview Browser
	[projectData setObject:[NSNumber numberWithInt:self.previewBrowser] forKey:@"preview-browser"];
	
	// General Files
	[projectData setObject:self.sourceFile forKey:@"source"];
	[projectData setObject:self.outputFile forKey:@"output"];
	[projectData setObject:self.launchFile forKey:@"launch"];
	
	// Flags
	NSMutableArray *array = [NSMutableArray new];
	for(TableItem *ti in [compileFlagsController arrangedObjects]) {
		[array addObject:[ti item]];
	}
	
	[projectData setObject:array forKey:@"flags"];
	[array release];
	
	// Save File
	[projectData writeToFile:self.projectFilePath atomically:YES];
	
}

////////////////////////////////////////
#pragma mark -
#pragma mark Paths
////////////////////////////////////////

- (NSURL*) urlToFileInProject:(NSString*)path {
	
	if ([path hasPrefix:@"http://"]) {
		return [NSURL URLWithString:path];
	} else {
		return [NSURL fileURLWithPath:[self absolutePathToFileInProject:path]];
	}
	
	return nil;
}

- (NSString*) absolutePathToFileInProject:(NSString*)path {
	
	if(![path isAbsolutePath] && ![path hasPrefix:@"http://"]) {
		NSString *projectPath = [self.projectFilePath stringByDeletingLastPathComponent];
		return [projectPath stringByAppendingPathComponent:path];
	}
	
	return path;
}

-(NSString*) projectDirectory {
	return [self.projectFilePath stringByDeletingLastPathComponent];
}

////////////////////////////////////////
#pragma mark -
#pragma mark Checks
////////////////////////////////////////

- (BOOL) isAIRProject {
	if ([self projectType] == AIR) {
		return YES;
	}
	
	return NO;
}

- (BOOL) isSourceFileAnFLA {
	return [[[self sourceFile] pathExtension] isEqual:@"fla"];
}

- (BOOL) fileExistsAtPath:(NSString*)file {
	
	if ([file length] > 0) {
		NSFileManager *fm = [NSFileManager defaultManager];
		
		if(![file isAbsolutePath]) {
			file = [[self.projectFilePath stringByAppendingFormat:@"/%@", file] stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
		}
		return [fm fileExistsAtPath:file];
	}
	
	return YES;
}

- (BOOL) settingsAreValid {	
	
	NSMutableString *message = [[NSMutableString new] autorelease];
	BOOL valid = YES;
	
	if ([self fileExistsAtPath:[self sourceFile]]) {
		if([self isAIRProject] && [self isSourceFileAnFLA]) {
			valid = NO;
			
			[message appendFormat:@"Project cannot be and AIR Application using an FLA source. Please set this up in Flash CS4"];
		}
	}
	
	
	if (!valid) {
		NSAlert *alert = [[NSAlert new] autorelease];
		
		[alert addButtonWithTitle:@"OK"];
		[alert setMessageText:@"Errors exist in project settings" ];
		[alert setAlertStyle:NSWarningAlertStyle];
		[alert setInformativeText:message];
		[alert beginSheetModalForWindow:window modalDelegate:self didEndSelector:@selector(alertDidEnd:returnCode:contextInfo:) contextInfo:nil];
		
	}
	
	
	
	return valid;
}

- (void)alertDidEnd:(NSAlert *)alert returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo {
	
}

////////////////////////////////////////
#pragma mark -
#pragma mark Field Accessors
////////////////////////////////////////

- (id)objectForKey:(NSString*)key {
	return [projectData objectForKey:key];
}

@synthesize projectTypePopUp;
@synthesize previewBrowserPopUp;
@synthesize configField;
@synthesize sourceField;
@synthesize outputField;
@synthesize launchField;
@synthesize compileFlagsController; 
@synthesize projectData;
@synthesize projectMenu;
 
- (NSInteger) projectType { return [[projectTypePopUp selectedItem] tag]; }
- (void) setProjectType:(NSInteger)value {
	[projectTypePopUp selectItemWithTag:value];
}

- (NSInteger) previewBrowser { return [[previewBrowserPopUp selectedItem] tag]; }
- (void) setPreviewBrowser:(NSInteger)value {
	[previewBrowserPopUp selectItemWithTag:value];
}


// config File
- (NSString*) configFile { return [configField stringValue]; }
- (void) setConfigFile:(NSString*)value {
	value = [value copy];
	[configField setStringValue:value];
}

// Source File
- (NSString*) sourceFile { return [sourceField stringValue]; }
- (void) setSourceFile:(NSString*)value {
	[sourceField setStringValue:[value copy]];
}

// Output File
- (NSString*) outputFile { return [outputField stringValue]; }
- (void) setOutputFile:(NSString*)value {
	value = [value copy];
	[outputField setStringValue:value];
}

// Launch File / URL
- (NSString*) launchFile { return [launchField stringValue]; }
- (void) setLaunchFile:(NSString*)value {
	value = [value copy];
	[launchField setStringValue:value];
}


// Compile Flags
- (NSArrayController*) flags {
	NSMutableArray *array = [NSMutableArray new];
	for(TableItem *ti in [compileFlagsController arrangedObjects]) {
		[array addObject:[ti item]];
	}
	
	return [array copy];
}

- (void) setFlags:(NSArrayController*)value {
	compileFlagsController = value;
}

////////////////////////////////////////
#pragma mark -
#pragma mark BWSheetControllerDelegate
////////////////////////////////////////

- (BOOL)shouldCloseSheet:(id)sender {
	
	NSArray *projectItems = [projectsList arrangedObjects];
	NSMutableArray *items = [NSMutableArray new];
	
	for(ProjectListItem *pl in projectItems) {
		if(pl.enabled = YES) {
			[items addObject:pl.filePath];
		}
	}

	[sugarDefaults setObject:items forKey:@"Projects"];
	[items release];
	
	return YES;
}

////////////////////////////////////////
#pragma mark -
#pragma mark Building
////////////////////////////////////////

- (IBAction) debugProject:(id)sender {
	
	if(compilerRunning) {
		[self stopBuildProcess];
	}
	[buildView clearMessages];
	[bsr reset];
	[self.mainTabView selectTabViewItemWithIdentifier:@"build"];
	
	if(buildTask) {
		[buildTask release];
		buildTask = nil;
	}
	
	buildTask = [[NSTask alloc] init];
	buildTask = [[NSTask alloc] init];
	[buildTask setStandardOutput: [NSPipe pipe]];
	[buildTask setStandardError: [buildTask standardOutput]];
	
	NSMutableArray *args = [[NSMutableArray alloc] initWithCapacity:2];
	NSString *sourceFile = [projectData objectForKey:@"source"];
	NSString *outputFile = [projectData objectForKey:@"output"];
	//NSString *directory = [self projectDirectory];
	NSArray *flags = [projectData objectForKey:@"flags"];
	
	/*
	if(![sourceFile isAbsolutePath]) {
		sourceFile = [directory stringByAppendingPathComponent:sourceFile];
	}
	
	if(![outputFile isAbsolutePath]) {
		outputFile = [directory stringByAppendingPathComponent:outputFile];
	}*/
	
	sourceFile = [self absolutePathToFileInProject:sourceFile];
	outputFile = [self absolutePathToFileInProject:outputFile];
	
	
	NSString *launchPath;
	
	
	if([self isSourceFileAnFLA]) {
		[args addObject: @"-e"];
		[args addObject: @"-s"];
		[args addObject:sourceFile];
		[args addObject: @"-o"];
		[args addObject:outputFile];
		
		launchPath = [[NSBundle mainBundle] pathForResource:@"flashcommand" ofType:nil];
		
	} else {
		
		[args addObject: [NSString stringWithFormat:@"%@", sourceFile]];
		[args addObject: [NSString stringWithFormat:@"-output=%@", outputFile]];
		
		if([self isAIRProject]) {
			[args addObject:[NSString stringWithString:@"+configname=air"]];
		}
		
		NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
		launchPath = [[userDefaults objectForKey:@"flexSDKPath"] stringByAppendingPathComponent:@"bin/mxmlc"];
	}

	
	if([flags count] > 0) {		
		for(NSString *flag in flags) {
			[args addObject:[flag stringByReplacingOccurrencesOfString:@"${PROJECT_PATH}" withString:[self projectDirectory]]];
		}
	}
	
		
	
	[[NSNotificationCenter defaultCenter] addObserver:self 
											 selector:@selector(getData:) 
												 name: NSFileHandleReadCompletionNotification 
											   object: [[buildTask standardOutput] fileHandleForReading]];
	
	[[[buildTask standardOutput] fileHandleForReading] readInBackgroundAndNotify];
	
	[buildTask setLaunchPath:launchPath];
	[buildTask setArguments:[args copy]];
	[buildTask launch];
	
	
	
	[args release];
	
	compilerRunning = YES;
}

- (void) stopBuildProcess {
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object: [[buildTask standardOutput] fileHandleForReading]];
    [buildTask terminate];
}

- (void) stopProcess {

    NSData *data;
	
	[self stopBuildProcess];
	
	while ((data = [[[buildTask standardOutput] fileHandleForReading] availableData]) && [data length]) {
		//[controller appendOutput: [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease]];
	}
	
	if(![bsr compileErrorCount])
		[self launchProject];
	
	compilerRunning = NO;
	
	//[self launchProcessEnded];
	//[buildTask release];
	//buildTask = nil;
}

- (void) getData: (NSNotification *)aNotification
{
    NSData *data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
	
	if ([data length])  {
        NSLog(@"MXMLC Build: %@", [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease]);
		
		NSString *message = [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease];
		
		[buildView addMessage:[bsr outputMessageFromData:data]];
		[buildView addLog:message];
		
    } else {
        [self stopProcess];
    }
    [[aNotification object] readInBackgroundAndNotify];  
}




////////////////////////////////
#pragma mark -
#pragma mark Running Project
////////////////////////////////

- (IBAction) runProject:(id)sender {
	[self launchProject];
}

- (void) launchProject {
	[self.mainTabView selectTabViewItemWithIdentifier:@"run"];
	
	if(![self isAIRProject]) {
		
		NSWorkspace *ws = [NSWorkspace sharedWorkspace];
		
		switch (self.previewBrowser) {
			case BUILTIN:
					[runView setPreviewDocument:[self urlToFileInProject: self.launchFile]];
				break;
			case EXTERNAL:
				[runView showDebugger];
				[ws openURL:[self urlToFileInProject: self.launchFile]];
				break;
		}
		
				
	} else {
		NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
		NSString *adlPath = [[userDefaults objectForKey:@"flexSDKPath"] stringByAppendingPathComponent:@"bin/adl"];
		
		NSArray *args = [NSArray arrayWithObject:[self absolutePathToFileInProject:self.launchFile]];
						 //[[self projectFilePath] stringByAppendingFormat:@"/%@", [self launchFile]]];
		
		NSLog(@"Launch Args %@:", args);
		
		[NSTask launchedTaskWithLaunchPath:adlPath arguments:args];
	}	
}

@end
